@@ -19,6 +19,7 @@ import json
 import time
 import warnings
 from scheduler import Scheduler, PENDING
+import configuration
 
 logger = logging.getLogger('luigi-interface')  # TODO: 'interface'?
 
@@ -35,6 +36,11 @@ class RemoteScheduler(Scheduler):
     def __init__(self, host='localhost', port=8082, connect_timeout=None):
         self._host = host
         self._port = port
+
+        config = configuration.get_config()
+
+        if connect_timeout is None:
+            connect_timeout = config.getfloat('core', 'rpc-connect-timeout', 10.0)
         self._connect_timeout = connect_timeout
 
     def _wait(self):
@@ -73,7 +79,7 @@ class RemoteScheduler(Scheduler):
         # just one attemtps, keep-alive thread will keep trying anyway
         self._request('/api/ping', {'worker': worker}, attempts=1)
 
-    def add_task(self, worker, task_id, status=PENDING, runnable=False, deps=None, expl=None):
+    def add_task(self, worker, task_id, status=PENDING, runnable=False, deps=None, expl=None, priority=0, family='', params={}):
         self._request('/api/add_task', {
             'task_id': task_id,
             'worker': worker,
@@ -81,6 +87,9 @@ class RemoteScheduler(Scheduler):
             'runnable': runnable,
             'deps': deps,
             'expl': expl,
+            'priority': priority,
+            'family': family,
+            'params': params,
         })
 
     def get_work(self, worker, host=None):
@@ -110,9 +119,15 @@ class RemoteScheduler(Scheduler):
     def task_list(self, status, upstream_status):
         return self._request('/api/task_list', {'status': status, 'upstream_status': upstream_status})
 
+    def task_search(self, task_str):
+        return self._request('/api/task_search', {'task_str': task_str})
+
     def fetch_error(self, task_id):
         return self._request('/api/fetch_error', {'task_id': task_id})
 
+    def add_worker(self, worker, info):
+        return self._request('/api/add_worker', {'worker': worker, 'info': info})
+
 
 class RemoteSchedulerResponder(object):
     """ Use on the server side for responding to requests
@@ -128,8 +143,11 @@ class RemoteSchedulerResponder(object):
     def __init__(self, scheduler):
         self._scheduler = scheduler
 
-    def add_task(self, worker, task_id, status, runnable, deps, expl, **kwargs):
-        return self._scheduler.add_task(worker, task_id, status, runnable, deps, expl)
+    def add_task(self, worker, task_id, status, runnable, deps, expl, priority=0, family='', params={}, **kwargs):
+        return self._scheduler.add_task(worker, task_id, status, runnable, deps, expl, priority, family, params)
+
+    def add_worker(self, worker, info, **kwargs):
+        return self._scheduler.add_worker(worker, info)
 
     def get_work(self, worker, host=None, **kwargs):
         return self._scheduler.get_work(worker, host)
@@ -151,6 +169,9 @@ class RemoteSchedulerResponder(object):
     def task_list(self, status, upstream_status, **kwargs):
         return self._scheduler.task_list(status, upstream_status)
 
+    def task_search(self, task_str, **kwargs):
+        return self._scheduler.task_search(task_str)
+
     def fetch_error(self, task_id, **kwargs):
         return self._scheduler.fetch_error(task_id)
 
