@@ -17,6 +17,7 @@ import json
 import os
 import atexit
 import mimetypes
+import posixpath
 import tornado.ioloop
 import tornado.netutil
 import tornado.web
@@ -99,8 +100,12 @@ class ByParamsHandler(BaseTaskHistoryHandler):
 
 class StaticFileHandler(tornado.web.RequestHandler):
     def get(self, path):
-        # TODO: this is probably not the right way to do it...
-        # TODO: security
+        # Path checking taken from Flask's safe_join function:
+        # https://github.com/mitsuhiko/flask/blob/1d55b8983/flask/helpers.py#L563-L587
+        path = posixpath.normpath(path)
+        if os.path.isabs(path) or path.startswith(".."):
+            return self.send_error(404)
+
         extension = os.path.splitext(path)[1]
         if extension in mimetypes.types_map:
             self.set_header("Content-Type", mimetypes.types_map[extension])
