@@ -14,6 +14,7 @@
 
 import random
 from scheduler import CentralPlannerScheduler, PENDING, FAILED, DONE
+import collections
 import threading
 import time
 import os
@@ -23,6 +24,7 @@ import traceback
 import logging
 import warnings
 import notifications
+import getpass
 from target import Target
 from task import Task
 
@@ -43,8 +45,11 @@ class Event:
     DEPENDENCY_DISCOVERED = "event.core.dependency.discovered"  # triggered for every (task, upstream task) pair discovered in a jobflow
     DEPENDENCY_MISSING = "event.core.dependency.missing"
     DEPENDENCY_PRESENT = "event.core.dependency.present"
+    BROKEN_TASK = "event.core.task.broken"
+    START = "event.core.start"
     FAILURE = "event.core.failure"
     SUCCESS = "event.core.success"
+    PROCESSING_TIME = "event.core.processing_time"
 
 
 class Worker(object):
@@ -57,9 +62,11 @@ class Worker(object):
 
     def __init__(self, scheduler=CentralPlannerScheduler(), worker_id=None,
                  worker_processes=1, ping_interval=None, keep_alive=None,
-                 wait_interval=None):
+                 wait_interval=None, max_reschedules=None):
+        self._worker_info = self._generate_worker_info()
+
         if not worker_id:
-            worker_id = 'worker-%09d' % random.randrange(0, 999999999)
+            worker_id = 'Worker(%s)' % ', '.join(['%s=%s' % (k, v) for k, v in self._worker_info])
 
         config = configuration.get_config()
 
@@ -75,6 +82,10 @@ class Worker(object):
                 wait_interval = config.getint('core', 'worker-wait-interval', 1)
             self.__wait_interval = wait_interval
 
+        if max_reschedules is None:
+            max_reschedules = config.getint('core', 'max-reschedules', 1)
+        self.__max_reschedules = max_reschedules
+
         self._id = worker_id
         self._scheduler = scheduler
         if (isinstance(scheduler, CentralPlannerScheduler)
@@ -86,9 +97,9 @@ class Worker(object):
         self.host = socket.gethostname()
         self._scheduled_tasks = {}
 
-        # store the previous tasks executed by the same worker
-        # for debugging reasons
-        self._previous_tasks = []
+        self.add_succeeded = True
+        self.run_succeeded = True
+        self.unfulfilled_counts = collections.defaultdict(int)
 
         class KeepAliveThread(threading.Thread):
             """ Periodically tell the scheduler that the worker still lives """
@@ -103,7 +114,7 @@ class Worker(object):
                 while True:
                     self._should_stop.wait(ping_interval)
                     if self._should_stop.is_set():
-                        logger.info("Worker was stopped. Shutting down Keep-Alive thread")
+                        logger.info("Worker %s was stopped. Shutting down Keep-Alive thread" % worker_id)
                         break
                     try:
                         scheduler.ping(worker=worker_id)
@@ -128,6 +139,24 @@ class Worker(object):
         self._keep_alive_thread.stop()
         self._keep_alive_thread.join()
 
+    def _generate_worker_info(self):
+        # Generate as much info as possible about the worker
+        # Some of these calls might not be available on all OS's
+        args = [('salt', '%09d' % random.randrange(0, 999999999))]
+        try:
+            args += [('host', socket.gethostname())]
+        except:
+            pass
+        try:
+            args += [('username', getpass.getuser())]
+        except:
+            pass
+        try:
+            args += [('pid', os.getpid())]
+        except:
+            pass
+        return args
+
     def _validate_task(self, task):
         if not isinstance(task, Task):
             raise TaskException('Can not schedule non-task %s' % task)
@@ -155,27 +184,34 @@ class Worker(object):
         notifications.send_error_email(subject, message)
 
     def add(self, task):
-        """ Add a Task for the worker to check and possibly schedule and run """
+        """ Add a Task for the worker to check and possibly schedule and run.
+         Returns True if task and its dependencies were successfully scheduled or completed before"""
+        self.add_succeeded = True
         stack = [task]
+        self._validate_task(task)
+        seen = set([task.task_id])
         try:
             while stack:
                 current = stack.pop()
                 for next in self._add(current):
-                    stack.append(next)
+                    if next.task_id not in seen:
+                        self._validate_task(next)
+                        seen.add(next.task_id)
+                        stack.append(next)
         except (KeyboardInterrupt, TaskException):
             raise
-        except:
+        except Exception as ex:
+            self.add_succeeded = False
             formatted_traceback = traceback.format_exc()
             self._log_unexpected_error(task)
+            task.trigger_event(Event.BROKEN_TASK, task, ex)
             self._email_unexpected_error(task, formatted_traceback)
+        return self.add_succeeded
 
     def _check_complete(self, task):
         return task.complete()
 
     def _add(self, task):
-        self._validate_task(task)
-        if task.task_id in self._scheduled_tasks:
-            return []  # already scheduled
         logger.debug("Checking if %s is complete", task)
         is_complete = False
         try:
@@ -184,6 +220,7 @@ class Worker(object):
         except KeyboardInterrupt:
             raise
         except:
+            self.add_succeeded = False
             formatted_traceback = traceback.format_exc()
             self._log_complete_error(task)
             task.trigger_event(Event.DEPENDENCY_MISSING, task)
@@ -192,25 +229,42 @@ class Worker(object):
             # failing complete()-method since we don't know if the task
             # is complete and subtasks might not be desirable to run if
             # they have already ran before
-            return []
+            return
 
         if is_complete:
-            # Not submitting dependencies of finished tasks
-            self._scheduler.add_task(self._id, task.task_id, status=DONE,
-                                      runnable=False)
+            deps = None
+            status = DONE
+            runnable = False
+
             task.trigger_event(Event.DEPENDENCY_PRESENT, task)
         elif task.run == NotImplemented:
-            self._add_external(task)
-        else:
-            return self._add_task_and_deps(task)
-        return []
+            deps = None
+            status = PENDING
+            runnable = False
 
-    def _add_external(self, external_task):
-        self._scheduled_tasks[external_task.task_id] = external_task
-        self._scheduler.add_task(self._id, external_task.task_id, status=PENDING,
-                                  runnable=False)
-        external_task.trigger_event(Event.DEPENDENCY_MISSING, external_task)
-        logger.warning('Task %s is not complete and run() is not implemented. Probably a missing external dependency.', external_task.task_id)
+            task.trigger_event(Event.DEPENDENCY_MISSING, task)
+            logger.warning('Task %s is not complete and run() is not implemented. Probably a missing external dependency.', task.task_id)
+
+        else:
+            deps = task.deps()
+            status = PENDING
+            runnable = True
+
+        if deps:
+            for d in deps:
+                self._validate_dependency(d)
+                task.trigger_event(Event.DEPENDENCY_DISCOVERED, task, d)
+                yield d # return additional tasks to add
+
+            deps = [d.task_id for d in deps]
+
+        self._scheduled_tasks[task.task_id] = task
+        self._scheduler.add_task(self._id, task.task_id, status=status,
+                                 deps=deps, runnable=runnable, priority=task.priority,
+                                 params=task.to_str_params(),
+                                 family=task.task_family)
+
+        logger.info('Scheduled %s (%s)', task.task_id, status)
 
     def _validate_dependency(self, dependency):
         if isinstance(dependency, Target):
@@ -218,21 +272,6 @@ class Worker(object):
         elif not isinstance(dependency, Task):
             raise Exception('requires() must return Task objects')
 
-    def _add_task_and_deps(self, task):
-        self._scheduled_tasks[task.task_id] = task
-        deps = task.deps()
-        for d in deps:
-            self._validate_dependency(d)
-            task.trigger_event(Event.DEPENDENCY_DISCOVERED, task, d)
-
-        deps = [d.task_id for d in deps]
-        self._scheduler.add_task(self._id, task.task_id, status=PENDING,
-                                  deps=deps, runnable=True)
-        logger.info('Scheduled %s', task.task_id)
-
-        for d in task.deps():
-            yield d  # return additional tasks to add
-
     def _check_complete_value(self, is_complete):
         if is_complete not in (True, False):
             raise Exception("Return value of Task.complete() must be boolean (was %r)" % is_complete)
@@ -240,21 +279,21 @@ class Worker(object):
     def _run_task(self, task_id):
         task = self._scheduled_tasks[task_id]
 
-        logger.info('[pid %s] Running   %s', os.getpid(), task_id)
+        logger.info('[pid %s] Worker %s running   %s', os.getpid(), self._id, task_id)
         try:
             # Verify that all the tasks are fulfilled!
-            ok = True
-            for task_2 in task.deps():
-                if not task_2.complete():
-                    ok = False
-                    missing_dep = task_2
-
-            if not ok:
-                # TODO: possibly try to re-add task again ad pending
-                raise RuntimeError('Unfulfilled dependency %r at run time!\nPrevious tasks: %r' % (missing_dep.task_id, self._previous_tasks))
-            task.run()
+            missing = [dep.task_id for dep in task.deps() if not dep.complete()]
+            if missing:
+                deps = 'dependency' if len(missing) == 1 else 'dependencies'
+                raise RuntimeError('Unfulfilled %s at run time: %s' % (deps, ', '.join(missing)))
+            task.trigger_event(Event.START, task)
+            t0 = time.time()
+            try:
+                task.run()
+            finally:
+                task.trigger_event(Event.PROCESSING_TIME, task, time.time() - t0)
             error_message = json.dumps(task.on_success())
-            logger.info('[pid %s] Done      %s', os.getpid(), task_id)
+            logger.info('[pid %s] Worker %s done      %s', os.getpid(), self._id, task_id)
             task.trigger_event(Event.SUCCESS, task)
             status = DONE
 
@@ -262,17 +301,38 @@ class Worker(object):
             raise
         except Exception as ex:
             status = FAILED
-            logger.exception("[pid %s] Error while running %s", os.getpid(), task)
+            logger.exception("[pid %s] Worker %s failed    %s", os.getpid(), self._id, task)
             error_message = task.on_failure(ex)
             task.trigger_event(Event.FAILURE, task, ex)
             subject = "Luigi: %s FAILED" % task
             notifications.send_error_email(subject, error_message)
 
         self._scheduler.add_task(self._id, task_id, status=status,
-                                  expl=error_message, runnable=None)
+                                 expl=error_message, runnable=None,
+                                 params=task.to_str_params(),
+                                 family=task.task_family)
 
+        # re-add task to reschedule missing dependencies
+        if missing:
+            reschedule = True
+
+            # keep out of infinite loops by not rescheduling too many times
+            for task_id in missing:
+                self.unfulfilled_counts[task_id] += 1
+                if self.unfulfilled_counts[task_id] > self.__max_reschedules:
+                    reschedule = False
+            if reschedule:
+                self.add(task)
+
+        self.run_succeeded &= status == DONE
         return status
 
+    def _add_worker(self):
+        try:
+            self._scheduler.add_worker(self._id, self._worker_info)
+        except:
+            logger.exception('Exception adding worker - scheduler might be running an older version')
+
     def _log_remote_tasks(self, running_tasks, n_pending_tasks):
         logger.info("Done")
         logger.info("There are no more tasks to run at this time")
@@ -286,6 +346,7 @@ class Worker(object):
         died_pid, status = os.wait()
         if died_pid in children:
             children.remove(died_pid)
+            self.run_succeeded &= status == 0
         else:
             logger.warning("Some random process %s died", died_pid)
 
@@ -309,8 +370,8 @@ class Worker(object):
         else:
             # need to have different random seeds...
             random.seed((os.getpid(), time.time()))
-            self._run_task(task_id)
-            os._exit(0)
+            status = self._run_task(task_id)
+            os._exit(0 if status == DONE else 1)
 
     def _sleeper(self):
         # TODO is exponential backoff necessary?
@@ -321,8 +382,13 @@ class Worker(object):
             yield
 
     def run(self):
+        """Returns True if all scheduled tasks were executed successfully"""
+
         children = set()
         sleeper  = self._sleeper()
+        self.run_succeeded = True
+
+        self._add_worker()
 
         while True:
             while len(children) >= self.worker_processes:
@@ -349,7 +415,6 @@ class Worker(object):
             else:
                 self._run_task(task_id)
 
-            self._previous_tasks.append(task_id)
-
         while children:
             self._reap_children(children)
+        return self.run_succeeded
