@@ -19,7 +19,8 @@ from luigi.mock import MockFile
 import mock
 import unittest
 import warnings
-
+import subprocess
+import os
 
 class SomeTask(luigi.Task):
     n = luigi.IntParameter()
@@ -50,12 +51,7 @@ class NonAmbiguousClass(luigi.Task):
         NonAmbiguousClass.has_run = True
 
 
-class DontRegisterThisOne(luigi.Task):
-    register_cls = False
-
-
 class TaskWithSameName(luigi.Task):
-    register_cls = False
     def run(self):
         self.x = 42
 
@@ -66,6 +62,17 @@ class TaskWithSameName(luigi.Task):
         self.x = 43
 
 
+class WriteToFile(luigi.Task):
+    filename = luigi.Parameter()
+    def output(self):
+        return luigi.LocalTarget(self.filename)
+
+    def run(self):
+        f = self.output().open('w')
+        print >>f, 'foo'
+        f.close()
+
+
 class CmdlineTest(unittest.TestCase):
     def setUp(self):
         global File
@@ -112,11 +119,13 @@ class CmdlineTest(unittest.TestCase):
     @mock.patch("warnings.warn")
     @mock.patch("luigi.interface.setup_interface_logging")
     def test_cmdline_logger(self, setup_mock, warn):
-        luigi.run(['Task', '--local-scheduler'])
-        self.assertEqual([mock.call(None)], setup_mock.call_args_list)
+        with mock.patch("luigi.interface.EnvironmentParamsContainer.env_params") as env_params:
+            env_params.return_value.logging_conf_file = None
+            luigi.run(['Task', '--local-scheduler'])
+            self.assertEqual([mock.call(None)], setup_mock.call_args_list)
 
         with mock.patch("luigi.configuration.get_config") as getconf:
-            getconf.return_value.get.return_value = None
+            getconf.return_value.get.side_effect = ConfigParser.NoOptionError(section='foo', option='bar')
             getconf.return_value.get_boolean.return_value = True
 
             luigi.interface.setup_interface_logging.call_args_list = []
@@ -127,13 +136,13 @@ class CmdlineTest(unittest.TestCase):
     def test_non_existent_class(self, print_usage):
         self.assertRaises(SystemExit, luigi.run, ['--local-scheduler', 'XYZ'])
 
-    @mock.patch('argparse.ArgumentParser.print_usage')
-    def test_not_registered_class(self, print_usage):
-        self.assertRaises(SystemExit, luigi.run, ['--local-scheduler', 'DontRegisterThisOne'])
-
-    def test_unregistered_class(self):
-        luigi.run(['--local-scheduler', 'TaskWithSameName'])
-        self.assertEqual(TaskWithSameName().x, 43)
+    def test_bin_luigi(self):
+        t = luigi.LocalTarget(is_tmp=True)
+        cmd = ['./bin/luigi', '--module', 'cmdline_test', 'WriteToFile', '--filename', t.path, '--local-scheduler']
+        env = os.environ.copy()
+        env['PYTHONPATH'] = env.get('PYTHONPATH', '') + ':.:test'
+        subprocess.check_call(cmd, env=env, stderr=subprocess.STDOUT)
+        self.assertTrue(t.exists())
 
 if __name__ == '__main__':
     unittest.main()
